import os

class Config(object):
    BOT_TOKEN = os.environ.get("BOT_TOKEN", None)
    API_HASH = os.environ.get("API_HASH", None)
    API_ID = int(os.environ.get("APP_ID", None))
    DB_URI = os.environ.get("DATABASE_URL", None)



import os
import telethon
import requests
from bot.config import Config
import functools
from telethon.errors import BadRequestError
from telethon.tl.functions.channels import EditAdminRequest, EditBannedRequest
from telethon.tl.functions.messages import UpdatePinnedMessageRequest
from telethon.tl.types import ChatAdminRights,ChatBannedRights,MessageEntityMentionName
from telethon import TelegramClient, events, functions, Button

#==================================================

bot = TelegramClient("bot", api_id=Config.API_ID, api_hash=Config.API_HASH)
utilsbot = bot.start(bot_token=Config.BOT_TOKEN)

#==================================================
def callback(sed):
    def callbacks(func):
        data = sed
        utilsbot.add_event_handler(
            func, events.callbackquery.CallbackQuery(data=data)
        )

    return callbacks

def cmd(add_cmd, is_args=False):
    def acmd(func):
        if is_args :
        	pattern = "^/" + add_cmd + "(?: |$)(.*)"
        elif is_args== "simple" :
            pattern = "^/" + add_cmd + " ?(.*)"
        elif is_args=="normal":
            pattern = "^/" + add_cmd
        else:
            pattern = "^/" + add_cmd + "$"
        utilsbot.add_event_handler(
            func, events.NewMessage(incoming=True, pattern=pattern)
        )
    return acmd

def inline():
    def ainline(func):
        utilsbot.add_event_handler(func, events.InlineQuery)
    return ainline

def is_admin():
    def decorator(func):
        @functools.wraps(func)
        async def wrapper(event):
            sed = await adminbot.get_permissions(event.chat_id, event.sender_id)
            user = event.sender_id
            if sed.is_admin:
                await func(event)
            if not user:
                pass
            if not sed.is_admin:
                await event.reply("Only Admins Can Use it.")
        return wrapper
    return decorator


def is_bot_admin():
    def decorator(func):
        @functools.wraps(func)
        async def wrapper(event):
            pep = await adminbot.get_me()
            sed = await adminbot.get_permissions(event.chat_id, pep)
            if sed.is_admin:
                await func(event)
            else:
                await event.reply("I Must Be Admin To Do This.")
        return wrapper
    return decorator


def only_groups():
    def decorator(func):
        @functools.wraps(func)
        async def wrapper(event):
            if event.is_group:
                await func(event)
            else:
                await event.reply("This Command Only Works On Groups.")
        return wrapper
    return decorator

def only_pm():
    def decorator(func):
        @functools.wraps(func)
        async def wrapper(event):
            if event.is_group:
                pass
            else:
                await func(event)

        return wrapper

    return decorator
#==================================================

is_admin = is_admin()
is_bot_admin = is_bot_admin()
only_groups = only_groups()
only_pm = only_pm()

# =================== CONSTANT ===================


from sqlalchemy import BigInteger, Boolean, Column, String, UnicodeText

from bot.sql import BASE, SESSION


class Welcome(BASE):
    __tablename__ = "welcome"
    chat_id = Column(String(14), primary_key=True)
    custom_welcome_message = Column(UnicodeText)
    media_file_id = Column(UnicodeText)
    should_clean_welcome = Column(Boolean, default=False)
    previous_welcome = Column(BigInteger)

    def __init__(
        self,
        chat_id,
        custom_welcome_message,
        should_clean_welcome,
        previous_welcome,
        media_file_id=None,
    ):
        self.chat_id = chat_id
        self.custom_welcome_message = custom_welcome_message
        self.media_file_id = media_file_id
        self.should_clean_welcome = should_clean_welcome
        self.previous_welcome = previous_welcome


Welcome.__table__.create(checkfirst=True)


def get_current_welcome_settings(chat_id):
    try:
        return SESSION.query(Welcome).filter(Welcome.chat_id == str(chat_id)).one()
    except:
        return None
    finally:
        SESSION.close()


def add_welcome_setting(
    chat_id,
    custom_welcome_message,
    should_clean_welcome,
    previous_welcome,
    media_file_id,
):
    # adder = SESSION.query(Welcome).get(chat_id)
    adder = Welcome(
        chat_id,
        custom_welcome_message,
        should_clean_welcome,
        previous_welcome,
        media_file_id,
    )
    SESSION.add(adder)
    SESSION.commit()


def rm_welcome_setting(chat_id):
    rem = SESSION.query(Welcome).get(str(chat_id))
    if rem:
        SESSION.delete(rem)
        SESSION.commit()


def update_previous_welcome(chat_id, previous_welcome):
    row = SESSION.query(Welcome).get(chat_id)
    row.previous_welcome = previous_welcome
    # commit the changes to the DB
    SESSION.commit()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

name = "utilsrobot"
version = "0.6"
author = "sppidy"
author_email = "sppidytg@gmail.com"
description = "A Powerful Python-Telethon Based Library For Spidy's Bots."
license = "GNU AFFERO GENERAL PUBLIC LICENSE (v3)"
url = "https://github.com/sppidy"

setuptools.setup(
    name=name,
    version=version,
    author=author,
    author_email=author_email,
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=url,
    license=license,
    packages=setuptools.find_packages(),
    install_requires=[
        "telethon==1.19.5",
        "cryptg",
        "python-dotenv==0.15.0",
        "redshift-sqlalchemy",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



import os

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session, sessionmaker

# the secret configuration specific things
from bot.config import Config

def start() -> scoped_session:
    engine = create_engine(Config.DB_URI)
    BASE.metadata.bind = engine
    BASE.metadata.create_all(engine)
    return scoped_session(sessionmaker(bind=engine, autoflush=False))


try:
    BASE = declarative_base()
    SESSION = start()
except AttributeError as e:
    # this is a dirty way for the work-around required for #23
    print(
        "DB_URI is not configured. Features depending on the database might have issues."
    )
    print(str(e))
